package be.iminds.ilabt.jfed.restapi.client;

import be.iminds.ilabt.jfed.fedmon.webapi.client.FedmonWebApiClient;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.*;
import be.iminds.ilabt.jfed.restapi.json.Experiment;
import be.iminds.ilabt.jfed.restapi.json.SpeaksForCredential;
import org.apache.http.client.utils.URIBuilder;

import javax.annotation.Nonnull;
import java.net.URI;
import java.util.*;

/**
 * FedmonWebApiClient: easy to use client for fedmon api
 */
public interface JFedRestApiClient {
    public static class JFedRestApiClientException extends Exception {
        public JFedRestApiClientException() {
        }

        public JFedRestApiClientException(String message) {
            super(message);
        }

        public JFedRestApiClientException(String message, Throwable cause) {
            super(message, cause);
        }

        public JFedRestApiClientException(Throwable cause) {
            super(cause);
        }

        public JFedRestApiClientException(String message, Throwable cause, boolean enableSuppression, boolean writableStackTrace) {
            super(message, cause, enableSuppression, writableStackTrace);
        }
    }

    default Optional<Experiment> getById(Class<? extends Experiment> objectClass, Long id) throws JFedRestApiClientException {
        return getById(objectClass, id, false);
    }
    Optional<Experiment> getById(Class<? extends Experiment> objectClass, Long id, boolean full) throws JFedRestApiClientException;
    Map<Long, Experiment> getAllIndexedById(Class<? extends Experiment> objectClass) throws JFedRestApiClientException;
    List<Experiment> getAll(Class<? extends Experiment> objectClass) throws JFedRestApiClientException;

    Experiment create(Experiment object) throws JFedRestApiClientException;
    Experiment update(Experiment object) throws JFedRestApiClientException;
    Experiment delete(Experiment object) throws JFedRestApiClientException;
    Experiment deleteById(Class<? extends Experiment> objectClass, Long id) throws JFedRestApiClientException;

    Optional<SpeaksForCredential> getSpeaksForCredential(@Nonnull String tld, @Nonnull String username) throws JFedRestApiClientException;
    void upsertSpeaksForCredential(@Nonnull String speaksForCredentialXmlString) throws JFedRestApiClientException;
    void upsertSpeaksForCredential(@Nonnull SpeaksForCredential object) throws JFedRestApiClientException;
    void deleteSpeaksForCredential(@Nonnull SpeaksForCredential object) throws JFedRestApiClientException;

    void deleteByUri(URI uri) throws JFedRestApiClientException;
}
